import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { PartnerRecommendation as PartnerRecommendation } from '../models/partner-recommendation';
import { PartnerRecommendationPagination as PartnerRecommendationPagination } from '../models/partner-recommendation-pagination';
import { PartnerRecommendationService as PartnerRecommendationService } from '../services/partner-recommendation.service';

import { ActivatedRoute, Router } from '@angular/router';

import { MatDialog } from '@angular/material/dialog';
import { RegistComposeComponent } from '../compose/regist-compose.component';
import { MatTableDataSource } from '@angular/material/table';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import { ListMembersResponse } from 'app/modules/proto/c2se/member_pb';
import { SelectionModel } from '@angular/cdk/collections';
import {
  AddComposeComponent,
  AddComposeResult,
} from '../compose/add-compose.component';
import { DeleteComposeComponent } from '../compose/delete-compose.component';
import { MemberService } from 'app/modules/polyglot/member/services/member.service';
import { MemberReferrerService } from 'app/modules/polyglot/member_referrer/services/member_referrer.service';
@Component({
  selector: 'partner-recommendation-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .partner-recommendation-grid {
        /*                     CB   상부   아이디 등급 계좌정보 */
        grid-template-columns: 20px 140px auto 100px 200px;

        @screen sm {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부*/
          grid-template-columns: 20px 140px auto 100px 200px 140px 200px 140px;
        }

        @screen md {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부  회원수 */
          grid-template-columns: 20px 140px auto 100px 200px 140px 200px 140px 140px;
        }

        @screen lg {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부 회원수 */
          grid-template-columns: 20px 100px auto 100px 200px 140px 200px 140px 140px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  registPartnerStoreDataSource: MatTableDataSource<MemberModel> =
    new MatTableDataSource();
  removePartnerStoreDataSource: MatTableDataSource<MemberModel> =
    new MatTableDataSource();

  registPartnerStoreTableColumns: string[] = [
    'select',
    'referrerMember',
    'memberClass',
    'username',
    'nickname',
    'amountMoney',
    'comp',
    'bankInfo',
    'state',
    'site',
  ];

  removePartnerStoreTableColumns: string[] = [
    'select',
    'referrerMember',
    'memberClass',
    'username',
    'nickname',
    'isUse',
    'memberCount',
  ];

  __isSearchOpened1 = false;
  __isSearchOpened2 = false;
  listMembers!: MemberModel[];

  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartnerRecommendation?: PartnerRecommendation;
  pagination?: PartnerRecommendationPagination;

  registSelection = new SelectionModel<MemberModel>(true, []);
  removeSelection = new SelectionModel<MemberModel>(true, []);

  addBtnConfigForm!: FormGroup;
  removeBtnConfigForm!: FormGroup;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _activatedRoute: ActivatedRoute,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _partnerRecommendationService: PartnerRecommendationService,
    private router: Router,
    private _matDialog: MatDialog,
    private _memberService: MemberService,
    private _memberReffererService: MemberReferrerService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the products
    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result =
        data['listPartnerStore'];

      this.listMembers = listMemberResult.getMembersList();

      this.registPartnerStoreDataSource.data = listMemberResult
        .getMembersList()
        .filter(
          (v: MemberModel) =>
            v.getMemberClass()?.getId() ===
            'e11cac11-3825-4f4e-9cd5-39367f23f973'
        );
      this.removePartnerStoreDataSource.data = listMemberResult
        .getMembersList()
        .filter(
          (v: MemberModel) =>
            v.getMemberClass()?.getId() ===
            'e11cac11-3825-4f4e-9cd5-39367f23f973'
        );

      /* console.log('dddd', listMemberResult.getMembersList()); */

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    /* if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the partnerRecommendation changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._partnerRecommendationService.getPartnerRecommendations(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    } */
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /* viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  } */
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  __registToggleAllRows() {
    if (this.__registIsAllSelected()) {
      this.registSelection.clear();
      return;
    }

    this.registSelection.select(...this.registPartnerStoreDataSource.data);
  }

  __registIsAllSelected() {
    const numSelected = this.registSelection.selected.length;
    const numRows = this.registPartnerStoreDataSource.data.length;
    return numSelected === numRows;
  }

  /** The label for the checkbox on the passed row */
  __registCheckboxLabel(row?: MemberModel): string {
    if (!row) {
      return `${this.__registIsAllSelected() ? 'deselect' : 'select'} all`;
    }
    return `${
      this.registSelection.isSelected(row) ? 'deselect' : 'select'
    } row ${row.getUsername()}`;
  }

  __removeToggleAllRows() {
    if (this.__removeIsAllSelected()) {
      this.removeSelection.clear();
      return;
    }

    this.removeSelection.select(...this.removePartnerStoreDataSource.data);
  }

  __removeIsAllSelected() {
    const numSelected = this.removeSelection.selected.length;
    const numRows = this.removePartnerStoreDataSource.data.length;
    return numSelected === numRows;
  }

  /** The label for the checkbox on the passed row */
  __removeCheckboxLabel(row?: MemberModel): string {
    if (!row) {
      return `${this.__removeIsAllSelected() ? 'deselect' : 'select'} all`;
    }
    return `${
      this.removeSelection.isSelected(row) ? 'deselect' : 'select'
    } row ${row.getUsername()}`;
  }

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch1(): void {
    this.__isSearchOpened1 = !this.__isSearchOpened1;
  }
  __onClickSearch2(): void {
    this.__isSearchOpened2 = !this.__isSearchOpened2;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickRegist(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(RegistComposeComponent);

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }

  __addBtnConfirmConfig(): void {
    this.addBtnConfigForm = this._formBuilder.group({
      title: '알림',
      message: '정말 추천인으로 추가하시겠습니까?',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: true,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }
  __removeBtnConfirmConfig(): void {
    this.removeBtnConfigForm = this._formBuilder.group({
      title: '알림',
      message: '정말 삭제하시겠습니까?',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: true,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }
  __onClickAddBtn(event: MouseEvent): void {
    // Open the dialog and save the reference of it
    const dialogRef = this._fuseConfirmationService.open();

    dialogRef.afterClosed().subscribe((result: AddComposeResult) => {
      console.log('Compose dialog was closed!');
      if (!!result.choice) {
        if (!!this.registSelection.hasValue()) {
          this.registSelection.selected.forEach((m: MemberModel) => {
            this._memberReffererService.createMemberReferrer(m.getId());
          });
        }
      }
    });
  }
  __onClickRemoveBtn(event: MouseEvent): void {
    // Open the dialog and save the reference of it
    const dialogRef = this._fuseConfirmationService.open();

    // Subscribe to afterClosed from the dialog reference
    dialogRef.afterClosed().subscribe((result) => {
      console.log(result);
    });
  }

  __getRefererMember(member: MemberModel): string {
    /* console.log(member.getReferrerMember()?.getUsername());

    const name = member.getReferrerMember()?.getUsername(); */
    return '없음';
  }
}
